@@ -1,3 +1,4 @@
+import traceback
 from datetime import datetime, timezone
 from typing import Optional
 from uuid import uuid4
@@ -10,7 +11,7 @@ import bson.errors
 from bson import ObjectId
 from pydantic import BaseModel, BaseConfig, Field
 
-from mongodm.types import ObjectIdStr
+from mongodm.types import ObjectIdStr, EncryptedStr, decrypt
 from mongodm.errors import InvalidSelection, NotFound, AbstractUsage
 
 
@@ -20,7 +21,11 @@ logger = logging.getLogger('mongodm')
 config = {
     'database_connection': AsyncIOMotorClient(),
     'database_name': 'database',
-    'soft_delete': False
+    'soft_delete': False,
+    'encryption_config': {
+        'public_key': '',
+        'private_key': ''
+    }
 }
 
 
@@ -31,6 +36,12 @@ def set_config(database_connection: AsyncIOMotorClient, database_name: str, soft
         'soft_delete': soft_delete
     })
 
+def set_encryption_config(public_key: str, private_key: str):
+    config['encryption_config'].update({
+        'public_key': public_key,
+        'private_key': private_key
+    })
+
 
 class MongODMBaseModel(BaseModel):
 
@@ -99,7 +110,10 @@ class MongoODMBase(MongODMBaseModel):
         if type(item) in [bytes, str, ObjectId, ObjectIdStr]:
             return cls.cast_to_object_id(item)
 
-        if type(item) in [list, dict]:
+        if type(item) in [list]:
+            return [cls.replace_str_with_object_id(i) for i in item]
+
+        if type(item) in [dict]:
             for key in item.keys():
                 if isinstance(item[key], list):
                     item[key] = [cls.replace_str_with_object_id(i) for i in item[key]]
@@ -107,6 +121,41 @@ class MongoODMBase(MongODMBaseModel):
                     item[key] = cls.replace_str_with_object_id(item[key])
         return item
 
+    @classmethod
+    def encrypt_encrypted_fields(cls, item):
+        if isinstance(item, EncryptedStr):
+            return item.encrypt(config['encryption_config']['public_key'])
+
+        if isinstance(item, list):
+            return [cls.encrypt_encrypted_fields(i) for i in item]
+
+        if isinstance(item, dict):
+            for key in item.keys():
+                if isinstance(item[key], list):
+                    item[key] = [cls.encrypt_encrypted_fields(i) for i in item[key]]
+                else:
+                    item[key] = cls.encrypt_encrypted_fields(item[key])
+        return item
+
+    @classmethod
+    def decrypt_encrypted_fields(cls, item):
+        if isinstance(item, bytes):
+            try:
+                return decrypt(item, config['encryption_config']['private_key'])
+            except Exception as e:
+                return item
+
+        if isinstance(item, list):
+            return [cls.decrypt_encrypted_fields(i) for i in item]
+
+        if isinstance(item, dict):
+            for key in item.keys():
+                if isinstance(item[key], list):
+                    item[key] = [cls.decrypt_encrypted_fields(i) for i in item[key]]
+                else:
+                    item[key] = cls.decrypt_encrypted_fields(item[key])
+        return item
+
     @classmethod
     def _get_fetch_filter(cls, selector):
         if not cls.__soft_delete__:
@@ -119,9 +168,10 @@ class MongoODMBase(MongODMBaseModel):
             to_exclude.add("created_at")
         if exclude:
             to_exclude = to_exclude.union(self.__protected_attributes__)
-        return self.replace_str_with_object_id(
-            self.dict(by_alias=True, exclude=to_exclude, exclude_none=exclude_none, exclude_unset=exclude_unset)
-        )
+        dump = self.dict(by_alias=True, exclude=to_exclude, exclude_none=exclude_none, exclude_unset=exclude_unset)
+        dump = self.encrypt_encrypted_fields(dump)
+        dump = self.replace_str_with_object_id(dump)
+        return dump
 
     async def before_create(self):
         """ Before create hook """
@@ -164,6 +214,7 @@ class MongoODMBase(MongODMBaseModel):
             raise InvalidSelection
         if item:
             e = cls(**item)
+            e.decrypt()
             await e.after_find()
             return e
         raise NotFound
@@ -184,6 +235,7 @@ class MongoODMBase(MongODMBaseModel):
             raise InvalidSelection
         if item:
             e = cls(**item)
+            e.decrypt()
             await e.after_find()
             return e
         raise NotFound
@@ -204,11 +256,18 @@ class MongoODMBase(MongODMBaseModel):
             raise InvalidSelection
         if item:
             e = cls(**item)
+            e.decrypt()
             await e.after_find()
             return e
         raise NotFound
 
+    def decrypt(self):
+        dump = self.dict(by_alias=True)
+        dump = self.decrypt_encrypted_fields(dump)
+        self.set_attributes(**dump)
+
     async def after_get_many_hook(self):
+        self.decrypt()
         await self.after_find()
         return self
 
@@ -246,6 +305,8 @@ class MongoODMBase(MongODMBaseModel):
         return [await cls(**item).after_get_many_hook() for item in items]
 
     def set_attributes(self, **kwargs):
+        if '_id' in kwargs.keys():
+            del kwargs["_id"]
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -262,8 +323,9 @@ class MongoODMBase(MongODMBaseModel):
             create = True
             return await self._create()
         await self.before_save()
-        self.updated_at = datetime.now()
         payload = self._get_dict_with_oid(exclude=True, exclude_none=exclude_none, exclude_unset=exclude_unset)
+        update_timestamp = datetime.now()
+        self.updated_at = payload['updated_at'] = update_timestamp
         if not create:
             payload = await self.before_update(payload)
         await config['database_connection'][config['database_name']][self.__collection_name__].update_one(
